/*
 * rx_add_on: Allows to add built-in functions
 */

#include <stdio.h>
#ifndef WNT
# include <sys/time.h>
#endif
#include <time.h>
#include "srexx.h"

#define NB  3

static ITEM *bu_left2(),
            *bu_pi2(),
            *bu_localtime();

static BUILT bb[NB] =
        { "left2"  , 0, bu_left2,
          "pi2"    , 0, bu_pi2,
          "localtime", 0, bu_localtime
        };


void rx_add_on()
{
  rx_add_on1(bb, NB);
}


/*
 * left2 is a copy of the LEFT() built-in function
 */
#include <math.h>    /* floor() */

ITEM *bu_left2(args)
  LEX *args;
{
  ITEM *res;
  ITEM **its;
  int nb_its, len1, len3, length, ll;
  register i;
  char *arg1, *arg3, *mes, pad, *mod;

 /*
  * We compute the supplied arguments
  */
  rx_nbfa(args, &its, &nb_its);

  if (  nb_its < 2 || nb_its > 3  || !its[0]  || !its[1] )
   { mes = strdupa("LEFT2(): must receive 2 or 3 arguments.");
     rx_bu_err(args, its, nb_its, 0, mes);
     /* NOTREACHED */
   }

  it2str(its[0], &arg1, &len1, ST1);

 /*
  * LENGTH
  */
  if (!str2db(its[1]))
   { mes = strdupa("LEFT2(): argument 2 must be a whole number.");
     rx_bu_err(args, its, nb_its, 1, mes);
     /* NOTREACHED */
   }
  its[1]->val.db = rx_round(its[1]->val.db, 9);
  if (its[1]->val.db < 0 || its[1]->val.db != floor(its[1]->val.db))
   { mes  = strdupa("LEFT2(): argument 2 must be a positive integer.");
     rx_bu_err(args, its, nb_its, 1, mes);
   }
  length = its[1]->val.db;


 /*
  * PAD
  */
  if (nb_its == 3  && its[2])
   { it2str(its[2], &arg3, &len3, ST3);
      if (len3 != 1)
       { mes = strdupa
              ("LEFT2(): the padding expression must be one character wide.");
         rx_bu_err(args, its, nb_its, 2, mes);
         /* NOTREACHED */
       }
     pad = *arg3;
   }
  else pad = ' ';

  mod = malloc_a(1+length);

  if (length)
   { ll = Min(length, len1);
     bcopy(arg1, mod, ll);
     for(i=ll; i<length; i++) mod[i] = pad;
   }

  mod[length] = '\0';

  res = (ITEM *)malloc_a(sizeof(res[0]));

  res->type        = IT_STR;
  res->val.str.str = mod;
  res->val.str.len = length;

  for (i=0; i<nb_its; i++) free_it(&its[i]);
  free(its);

  return(res);
}

/*
 * pi returns PI with arg(1) decimals
 */

static char *buff;
static ind;

static void add(v)
 int v;
{
  sprintf(buff+ind, "%.4d", v);

  if (ind) ind += 4;
  else
   { bcopy(buff+1, buff+2, 3);
     buff[1] = '.';
     ind += 5;
   }
}


ITEM *bu_pi2(args)
  LEX *args;             /* ARGSUSED*/
{
  ITEM *res, **its;
  register b, c, d, e, g;
  int *f, len, bl, nb_its;
  double d_bl;
  char *mes;

 /*
  * We compute the supplied argument
  */
  rx_nbfa(args, &its, &nb_its);

  if (nb_its != 1 || !its[0] )
   { mes = strdupa("PI(): must receive 1 argument.");
     rx_bu_err(args, its, nb_its, 0, mes);
     /* NOTREACHED */
   }

  if ( !str2db(its[0]) ||
       its[0]->val.db <= 0 || its[0]->val.db != floor(its[0]->val.db)
     )
   { mes = strdupa("PI(): argument 1 must be a positive number.");
     rx_bu_err(args, its, nb_its, 0, mes);
     /* NOTREACHED */
   }

  len  = its[0]->val.db;
  d_bl = (double)len/4.0;
  bl   = ceil(d_bl);
  c    = 14*bl;
  len  = 4*bl;
  f    = (int *) malloc_a((1+c)*sizeof(f[0]));
  buff = malloc_a(len+2);

  for(b=0; b<c; b++) f[b] = 2000;
  f[b] = 0;

  e = ind = 0;

  for(;d=0, g = c*2; c-=14, add(e+d/10000), e=d%10000)
    for(b=c; d+=f[b]*10000, f[b]=d%--g, d/=g--, --b; d*=b);

  free(f);

  res = (ITEM *)malloc_a(sizeof(res[0]));

  len = its[0]->val.db;
  buff[1 + len] = '\0';

  if (len > 15)
   { /*
      * We return a string
      */
     res->type        = IT_STR;
     res->val.str.str = buff;
     res->val.str.len = len+1;
   }
  else
   { /*
      * We return a double
      */
     res->type  = IT_DB;
     res->prec  = 0;
     sscanf(buff, "%lf", &res->val.db);
     free(buff);
   }

  return(res);

}


/*
 * locatime: localtime accept argument representing the time in
 *           seconds  since  00:00:00  UTC,  January  1,  1970.
 *           Returns a string in hh:mm:ss dd:mm:yyyy format
 */

ITEM *bu_localtime(args)
  LEX *args;
{
  ITEM *res;
  ITEM **its;
  int nb_its;
  register i;
  char *mes, buff[100];
  time_t sec;
  struct tm *dh;

 /*
  * We compute the supplied arguments
  */
  rx_nbfa(args, &its, &nb_its);

  if (  nb_its != 1  || !its[0] )
   { mes = strdupa("LOCALTIME(): must receive 1 argument.");
     rx_bu_err(args, its, nb_its, 0, mes);
     /* NOTREACHED */
   }

  if (!str2db(its[0]))
   { mes = strdupa("LOCALTIME(): argument 1 must be a whole number.");
     rx_bu_err(args, its, nb_its, 1, mes);
     /* NOTREACHED */
   }

  its[0]->val.db = rx_round(its[0]->val.db, 9);
  if (its[0]->val.db < 0 || its[0]->val.db != floor(its[0]->val.db))
   { mes  = strdupa("LOCALTIME(): argument 1 must be a positive integer.");
     rx_bu_err(args, its, nb_its, 1, mes);
   }
  sec = its[0]->val.db;
  dh = localtime(&sec);
  sprintf(buff, "%2.2d:%2.2d:%2.2d  %2.2d/%2.2d/%4.4d", dh->tm_hour,
          dh->tm_min, dh->tm_sec,
          dh->tm_mday, dh->tm_mon+1, dh->tm_year+1900);



  res = (ITEM *)malloc_a(sizeof(res[0]));

  res->type        = IT_STR;
  res->val.str.str = strdupa(buff);
  res->val.str.len = strlen(buff);

  for (i=0; i<nb_its; i++) free_it(&its[i]);
  free(its);

  return(res);
}
