DY_OPEN (UNIX) - Displays the content of a directory
----------------------------------------------------


See below for the WINDOWS DY_OPEN description.

rc = DY_OPEN(x, y, ht, stem, {dir}, {filt_f}, {filt_d}, {label}, {lg_reg}
             {, Rb, Gb, Bb}))

rc      is the return code. It will be set to 0 when the requested
        directory has been successfully scanned.

x       is the dialog box column position. When x is set to 0, the dialog
        box is horizontally centered regarding the screen when S/REXX is
        used as a UNIX script, or centered regarding the SEDIT window
        position when S/REXX is used as an SEDIT macro.

y       is the dialog box line position. When y is set to 0, the dialog
        box is vertically centered regarding the screen when S/REXX is
        used as a UNIX script, or centered regarding the SEDIT window
        position when S/REXX is used as an SEDIT macro.

ht      is the height of the file and directories scrolling lists.

stem    must be a valid S/REXX variable name. stem.0 will contain the
        number of selected files. stem.1, stem.2, etc..., will contain the
        names of the selected files.

dir     is the initial directory to be scanned. When omitted, the current
        directory will be scanned.

filt_f  is the filter used to select which files are to be displayed. When
        omitted, it defaults to *, which means any file. When the regular
        expression switch is off, * means any set of characters. a*df*
        would for example match a_123.dfte.

filt_d  is the filter used to select which directories are to be
        displayed.

label   is the label to be displayed on the top of the dialog box.

lg_reg  when set to 1, toggles on the regular expression search. When
        omitted, or set to 0, toggles off the regular expression search.


Rb, Gb, Bb  are the background color RGB values. These are integers
            between 0 and 255. The default MOTIF background color can be
            set with the dialog*background X11 resource.

A regular expression is a character string used to describe, in an
extended way, a string to be matched. It uses special characters, called
meta characters to describe the match to be done.

The meta characters are the following:

^       matches only at the beginning of a line.
$       matches only at the end of a line.
\<      matches only at the beginning of a word.
\>      matches only at the end of a word.
.       matches any single character.
[]      matches any character in a character class.
\(      delimits the start of a subexpression. It is available for VI
        compatibility, but has no special meaning.
\)      delimits the end of a subexpression. It is available for VI
        compatibility, but has no special meaning.
*       repeats the preceding 0 or more times.

If you want to use a meta character as an ordinary character, you must
precede it with a backslash (\) character.


Examples:

------------------------------------------------------------------------
^af
------------------------------------------------------------------------
matches the string "af" only at the beginning of a line.

------------------------------------------------------------------------
af$
------------------------------------------------------------------------
matches the string "af" only at the end of a line.

------------------------------------------------------------------------
dc.......is
------------------------------------------------------------------------
matches the string "dc" followed by any 7 characters followed by the
string "is".


------------------------------------------------------------------------
[A-Z][a-z]
------------------------------------------------------------------------
[A-Z]   means any character from A to Z.
[a-z]   means any character from a to z.
The whole expression above matches any alphabetical string starting with a
capital letter.
The string "File001" will be matched. "F001" will not.

Note that the meta characters are not treated when enclosed in brackets:

------------------------------------------------------------------------
[.$]
------------------------------------------------------------------------
matches the string ".$". Without brackets, the user should type:

------------------------------------------------------------------------
\.\$
------------------------------------------------------------------------
for the same match.


Supported user actions
----------------------

The user can perform the following actions:

Clicking once with the left mouse button on a displayed file selects this
file, and displays its name in the file input zone.

With the MOTIF version, holding the Shift key down while clicking extends
the selection to several contiguous files. Holding the Control key down
extends the selection to another, possibly non-contiguous, file.

Double clicking on a file makes DY_OPEN return. stem will contain the name
of all the selected files.

Clicking once with the left mouse button upon a displayed directory
selects this directory, and displays its name in the directory input zone.

Double clicking on a displayed directory initiates a scan of the
directory.

Using the Return or Enter key when the keyboard focus is in the File input
zone makes DY_OPEN return. stem will contain the name of the file
displayed in that File input zone.

Using the Return or Enter key when the keyboard focus is in the Directory
input zone initiates a scan of the directory displayed in that zone. When
this directory name does not start with a / or a ~, it will be considered
as a subdirectory of the previously scanned directory.

Using the Return or Enter key when the keyboard focus is in the File or
Directory filter zone initiates a new scan of the previously scanned
directory.

Clicking on the OPEN button makes DY_OPEN return. stem will contain the
name of all the selected files.

Clicking on the CANCEL button makes DY_OPEN return. stem.0 will contain
the 0 string.

Clicking on the SCAN button initiates a new scan of the previously scanned
directory.


The following /home/xed/xmac/dy_open.sedit macro is used by the SEDIT text
editor to open new files:

--------------------------------------------------------------------------
/*
 * dy_open: opens a set of new files
 *
 *  Syntax:  DY_OPEN  {dir}, {filt-file}, {filt-dir}, {lg_reg}
 *
 *   dir:   The directory to scan, or //last to open the last directory
 *          scanned
 *
 *  You may use this macro without an S/REXX license, providing you do not
 *  modify it
 */

signal on novalue
option mixed setenv

parse arg dir, ff, fd, reg

if dir = '' then dir = cwd()
else
 if dir = '//last' then
  { if $?last_dy_open_scanned then dir = $last_dy_open_scanned
    else                           dir = '.'
  }

if ff = '' then ff = '*'
if fd = '' then fd = '*'
if reg = '' then reg = 0

trace off
call dy_open 0, 0, 10, sfi, dir, ff, fd, "xed - dir:"cwd(), reg

if result = 0 then
 { do i = 1 to sfi.0
    'xed 'sfi.i
   end i
   if sfi.0 ~= 0 then $last_dy_open_scanned = fd(sfi.1)
 }
else 'prompt Unable to scan 'dir
--------------------------------------------------------------------------




DY_OPEN (WINDOWS)  - Displays the content of a directory
--------------------------------------------------------

rc = DY_OPEN(x, y, ht, stem, {dir}, {filt_f}, {filt_d}, {label}, {lg_reg})


DY_OPEN displays the standard WINDOWS dialog box.

rc      is the return code. It will be set to 0 when the requested
        directory has been successfully scanned.

x       unused.

y       unused.

ht      unused.

stem    must be a valid S/REXX variable name. stem.0 will contain the
        number of selected files. stem.1, stem.2, etc..., will contain the
        names of the selected files.

dir     is the initial directory to be scanned. When omitted, the current
        directory will be scanned.

filt_f  is the filter used to select which files are to be displayed. When
        omitted, it defaults to *, which means any file.

filt_d  unused.

label   is the label to be displayed on the top of the dialog box.

lg_reg  unused.
