GLOBALV - Share Variables


GLOBALV is not part of the S/REXX language, but for VM/CMS compatibility
it is implemented as an extension of the UNIX or WINDOWS environment.
Therefore, it is recommended to enclose it in quotes in order to prevent
the normal S/REXX variable substitution.

When OPTION NOGLOBALV is in effect, the initial processing relative to the
GLOBALV command does not take place, and GLOBALV will not be recognized as
an extension to the UNIX, WINDOWS or SEDITenvironment.

GLOBALV format is:

globalv |init
        |
        |{select        {grp}}
        |
        |{select        {grp}}  set     name1 {val1 name2{....}}
        |{select        {grp}}  sets    name1 {val1 name2{....}}
        |{select        {grp}}  setp    name1 {val1 name2{....}}
        |
        |{select        {grp}}  setl    name {val}
        |{select        {grp}}  setls   name {val}
        |{select        {grp}}  setsl   name {val}
        |{select        {grp}}  setlp   name {val}
        |{select        {grp}}  setpl   name {val}
        |
        |{select        {grp}}  list    {name1 {name2 ...}}
        |{select        {grp}}  stack   {name1 {name2 ...}}
        |
        |{select        {grp}}  put     name1 {name2 {...}}
        |{select        {grp}}  puts    name1 {name2 {...}}
        |{select        {grp}}  putp    name1 {name2 {...}}
        |
        |{select        {grp}}  get     {name1 {name2 ...}}}
        |
        |
        |{select        {grp}}  purge
        |
        |grplist
        |grpstack

An S/REXX script, called the parent, can start several other S/REXX
scripts, either by using the UNIX or WINDOWS environment, or the CALL
command when OPTION NOLOAD is in effect.

The GLOBALV command lets an S/REXX script and its descendants share a
common set of values, and also retain them for subsequent use by other
parents.

GLOBALV maintains groups of variables in the GLOBALDIR/srexx.globalv
directory. By default, GLOBALDIR is the home ~ directory on UNIX, and the
C:\ directory on WINDOWS.

GLOBADIR can be changed by creating a GLOBALVHOME environment variable.
That can be done at the start of the parent S/REXX script:

call setenv "GLOBALVDIR", "/tmp"

GLOBALV supports use of more than one group. This allows for grouping
distinct variables that are either related or often used together.

Variables defined for the current parent S/REXX script are retained in the
GLOBALDIR/srexx.globalv/storage.xxxx directory, where xxxx is unique to
the parent script. Those required longer than a single parent existence
are retained in the GLOBALDIR/srexx.globalv/session and
GLOBALDIR/srexx.globalv/lasting files.

These two files and a third file (GLOBALDIR/srexx.globalv/initial) are the
source from which GLOBALV creates and initializes the variable or
variables in the storage.xxxx file. The initial file is normally created
by the user as an alternative way of defining a large number of initial
variables.

init    allocates and initializes global variable or variables in the
        storage.xxxx file from the variables stored in the lasting,
        session, and initial files. Variables defined in the session file
        override identical variables defined in the lasting file, which
        override identical variables defined in the initial file.

        GLOBALV INIT is performed automatically if not explicitly
        requested before other GLOBALV requests.

select {grp}

        identifies the global variable group that is the subject of this
        or subsequent calls. If no function is specified, GLOBALV sets the
        default group for subsequent calls. The default is set to the grp
        group or to unnamed if grp is not specified. A GLOBALV SELECT
        command that does specify a function affects only the group
        specified in the command. It has no effect on setting or resetting
        the default group.

SET, SETS, SETP name1 {{val1 name2{....}}

        assigns the vali values to the namei variables. SET fields are
        delimited by blanks and the values cannot contain any blanks. (Use
        the SETL command for such values.) If vali is not specified, the
        value is assumed to be an empty string. SET adds the assignments
        in the selected or default global variable group in storage. SETS
        adds or replaces the assignments in the selected or default group
        and appends it to the session file. SETP adds or replaces the
        assignments in the selected or default group and appends it to the
        lasting file.

SETL, SETLS, SETSL, SETLP, SETPL name {val}

        assigns the specified literal val value, which may contain blanks,
        to the name variable. The first blank following the name delimits
        the name from the value field and is not part of the value. All
        characters following this blank (including any other blanks) are
        part of the value. If val is not specified, the value is assumed
        to be an empty string.

        SETL adds the assignment in the selected or default global
        variable group in storage. SETLS adds the assignment in the
        selected or default group and appends it to the session file.
        SETSL is the same as SETLS. SETLP adds the assignment in the
        selected or default group and appends it to the lasting file.
        SETPL is the same as SETLP.

LIST    displays a list of the specified variable name or names, from the
        selected or default group, and their associated values. If no name
        is specified, all variables in the selected or default group are
        listed.

STACK   places the values associated with the specified variable name or
        names, from the selected or default group, LIFO in the program
        stack. The variable named first in the command is the first
        retrieved from the stack. If a variable is not found in the group,
        an empty string is stacked. The command has no effect if the
        variable name is omitted.

PUT, PUTS, PUTP

        PUT assigns the value of the S/REXX variable specified in name as
        a global value in the selected or default global variable group.
        PUTS does the same, and appends the value to the session file.
        PUTP does the same, but appends the value to the lasting file.

        Usage note: to be compatible with its VM/CMS implementation, the
                    various PUT commands do not perform a complete stem
                    substitution.

        For example:

             tab.m = 'Value of tab.m'
             tab.k = 'Value of tab.k'
             k = 'M'             /* k = 'm' with OPTION MIXED */
             say tab.k           /* Displays 'Value of tab.m' */
             'globalv put tab.k' /* Saves 'Value of tab.k'    */

GET     assigns values from the specified or default global variable group
        to the specified S/REXX variable names. If no names are specified,
        GET does nothing.

        Usage note: to be compatible with its VM/CMS implementation, GET
                    does not perform a complete stem substitution. See the
                    example above.

PURGE   clears the variables from the selected group in storage.xxxx. Used
        without SELECT, PURGE clears all variables in all groups.

GRPLIST displays a list of all groups.

GRPSTACK

        stacks LIFO the names of all groups. An empty string delimiter
        indicates the end of the stacked names.

The GLOBALV return codes have the following meanings:

     0       Success.
     1000024 No Function Specified on GLOBALV command.
     1100000 I/O Error reading GLOBALV type files.
     1500000 Invalid HOME or GLOBALVHOME.
     1501000 I/O Error on initial.
     1502000 I/O Error on lasting.
     1503000 I/O Error on session.
     1504000 I/O Error on storage.
     1000004 Invalid argument.
     1505000 Unable to lock file.


Examples:

'globalv select grp1 set var1 val_var1 var2 val_var2 var3'

places the 'val_vari' strings into the grp1 group vari variables. The
previous default group is not changed.

'globalv select grp1'

sets grp1 as the default group.

'globalv get var2'

retrieves into the S/REXX var2 variable the previously saved val_var2
value.
