PARG - Parse argument


parg(argv, opt, tab) parses the argv argument string according to the
options defined by the opt string, saving the result into the tab stem.

An option is a string starting with a minus sign. A parameter is a string
following an option.

parg returns 0 when the argument string argv matches the options, 1 when
the number of parameters following any option does not match the number of
parameters defined by opt and 2 when an option is not defined.

Consider the following test program:


-----------------------------------------------------------------------
#! /home/xed/srexx
option mixed
opt = "-Wp 2 -help 0 -font 1"
parse arg argv
if parg(argv, opt, tab) ~= 0 then
 { say 'Invalid option.'
   exit 1
 }
do i = 1 to tab.0
  select
    when tab.i = 'help'   then call help_proc
    when tab.i = 'font'   then
     { font = tab.font.1
       say 'Font set to "'font'"'
     }
    when tab.i = 'Wp'     then
     { px = tab.Wp.1
       py = tab.Wp.2
       say 'Position set to "'px py'"'
     }
  end
end i
str = tab.i
say 'Final string: "'str'"'
exit

help_proc:
  say "Help not yet available"
  exit 1
-----------------------------------------------------------------------

The "-Wp 2 -help 0 -font 1" opt string has the following meaning:

  The -Wp option must be followed by 2 parameters.
  The -help option must be followed by no parameter.
  The -font option must be followed by 1 parameter.

tab.0 gives the number of recognized options passed to the program.
If n is tab.0+1, tab.n gives the remaining argument string.

When the -Wp option is passed to the program, tab.Wp.1 and tab.Wp.2 give
the two words following the -Wp option in the argv argument string.


Examples:


-----------------------------------------------------------------------
% test -Wp 345 123 str1 str2 str3
Position set to "345 123"
Final string: "str1 str2 str3"
% test -help
Help not yet available
% test -font cour.b.18
Font set to "cour.b.18"
Final string: ""
%
-----------------------------------------------------------------------


When option mixed is not in effect, all options are translated into
uppercase. In this case, the test program would be:


-----------------------------------------------------------------------
#! /home/xed/srexx
opt = "-wp 2 -help 0 -font 1" /* Same as "-WP 2 -HELP 0 -FONT 1" */
parse arg argv
if parg(argv, opt, tab) ~= 0 then
 { say 'Invalid option.'
   exit 1
 }
do i = 1 to tab.0
  select
    when tab.i = 'HELP'   then call help_proc
    when tab.i = 'FONT'   then
     { font = tab.font.1
       say 'Font set to "'font'"'
     }
    when tab.i = 'WP'     then
     { px = tab.wp.1
       py = tab.wp.2
       say 'Position set to "'px py'"'
     }
  end
end i
str = tab.i
say 'Final string: "'str'"'
exit

help_proc:
  say "Help not yet available"
  exit 1
-----------------------------------------------------------------------
